from flask import Flask, session, render_template, request, send_from_directory, jsonify, make_response, render_template_string
import os
import secrets
import string
import mysql.connector
import re
import multiprocessing

app = Flask(__name__)
app.secret_key = '<REDACTED>'
app.config['MYSQL_HOST'] = 'localhost'
app.config['MYSQL_USER'] = 'root'
app.config['MYSQL_PASSWORD'] = '<REDACTED>'
app.config['MYSQL_DB'] = 'chamb'

def get_db_connection():
    return mysql.connector.connect(
        host=app.config['MYSQL_HOST'],
        user=app.config['MYSQL_USER'],
        password=app.config['MYSQL_PASSWORD'],
        database=app.config['MYSQL_DB']
    )
def generate_random_voucher():
    characters = string.ascii_letters
    digits = string.digits
    voucher = "zoz"
    voucher += secrets.choice("-_.")
    voucher += ''.join(secrets.choice(characters) for i in range(random.randint(3, 5)))
    voucher += secrets.choice("-_.")
    voucher += ''.join(secrets.choice(digits) for i in range(random.randint(6, 12)))
    return voucher

def generate_random_username(length=16):
    characters = string.ascii_letters + string.digits
    return ''.join(secrets.choice(characters) for i in range(length))

@app.route('/', methods=['GET'])
def index():
    conn = get_db_connection()
    cursor = conn.cursor()
    if session.get('username') is not None:
        
        cursor.execute('SELECT first_time FROM users WHERE username = %s', (session.get('username'),))
        first_time = cursor.fetchone()[0]
        cursor.execute('SELECT balance FROM users WHERE username = %s', (session.get('username'),))
        balance = cursor.fetchone()[0]
        cursor.close()
        conn.close()
        if first_time == 0:
            return render_template('index.html', voucher=f"Your first time $100 voucher: {session.get('voucher')}",balance=f"${balance}")
        return render_template('index.html', balance=f"${balance}")
    username = generate_random_username()
    voucher = generate_random_voucher()
    session['voucher'] = voucher
    session['username'] = username
    cursor.execute('INSERT INTO users (username) VALUES (%s)', (username,))
    conn.commit()
    cursor.close()
    conn.close()

    return render_template('index.html', voucher=f"Your first time $100 voucher: {session.get('voucher')}",balance=f"$0")

def redeem_voucher_process(username, voucher, result):
    conn = get_db_connection()
    cursor = conn.cursor()
    cursor.execute('SELECT first_time FROM users WHERE username = %s', (username,))
    first_time = cursor.fetchone()[0]

    if first_time == 1:
        result['response'] = ("You can redeem your voucher only once", 400)
        cursor.close()
        conn.close()
        return
    conn = get_db_connection()
    cursor = conn.cursor()
    try:
        pattern = r"^zoz[-_.][a-zA-Z0-9]{3,5}[-_.](\d+)+$"
        cursor.execute('UPDATE users SET balance = balance + 100 WHERE username = %s', (username,))
        conn.commit()
        print("hereee")
        match = re.match(pattern, voucher)
        print(voucher)
        if match:
            print("Voucher matched")
            cursor.execute('UPDATE users SET first_time = 1 WHERE username = %s', (username,))
            conn.commit()
            result['response'] = ("Voucher redeemed, please refresh the page to view your balance", 200)
        else:
            cursor.execute('UPDATE users SET balance = balance - 100 WHERE username = %s', (username,))
            conn.commit()
            result['response'] = ("Invalid voucher", 400)
    except Exception as e:
        result['response'] = (str(e), 400)
    finally:
        cursor.close()
        conn.close()

@app.route('/redeem_voucher', methods=['POST'])
def redeem_voucher():
    username = session.get('username')
    if not username:
        return "User not logged in please refresh the page", 400

    voucher = request.form.get('voucher')
    if not voucher:
        return "Voucher not provided", 400

    result = multiprocessing.Manager().dict()
    process = multiprocessing.Process(target=redeem_voucher_process, args=(username, voucher, result))
    process.start()

    timeout = 1  
    process.join(timeout)

    if process.is_alive():
        process.terminate()
        process.join()
        return "Something Went wrong", 400

    return result['response'][0], result['response'][1]

@app.route('/buy_product', methods=['GET'])
def buy_product():
    products = {
        'Drill': 75,
        'Pliers': 10,
        'Wrench': 30,
        'Hard hat': 50,
        'Screwdriver': 15,
        'Paintbrush': 5,
        'Hammer': 25,
        'Flag': 1337  
    }

    username = session.get('username')
    if not username:
        return jsonify({"error": "User not logged in"}), 401

    product_name = request.args.get('product')
    if product_name not in products:
        return jsonify({"error": "Product not found"}), 404

    product_price = products[product_name]

    conn = get_db_connection()
    cursor = conn.cursor()

    cursor.execute('SELECT balance FROM users WHERE username = %s', (username,))
    current_balance = cursor.fetchone()[0]

    if current_balance < product_price:
        cursor.close()
        conn.close()
        return jsonify({"error": "Insufficient balance"}), 400

    new_balance = current_balance - product_price

    cursor.execute('UPDATE users SET balance = %s WHERE username = %s', (new_balance, username))
    conn.commit()

    cursor.close()
    conn.close()
    if product_name == "Flag":
        return jsonify({
            "message": f"Wow how do you have so much money here is your flag: {os.environ.get('FLAG')}"
        }), 200

    return jsonify({
        "message": f"Successfully purchased {product_name}. Remaining balance: {new_balance}, We will ship to you as fast as possible :)"
    }), 200

@app.route('/css/<path:filename>')
def serve_css(filename):
    return send_from_directory(os.path.join(app.root_path, 'assets', 'css'), filename)

@app.route('/js/<path:filename>')
def serve_js(filename):
    return send_from_directory(os.path.join(app.root_path, 'assets', 'js'), filename)

@app.route('/images/<path:filename>')
def serve_images(filename):
    return send_from_directory(os.path.join(app.root_path, 'assets', 'images'), filename)

if __name__ == '__main__':
    app.run("0.0.0.0", debug=False)
